import { FC, ReactNode } from "react";
import { StyleProp, ViewStyle } from "react-native";
import { Colors } from "../../theme";
import { TestProps } from "../../util";
import { ButtonVariant } from "../button";
/** Specifies what type of dialog to render */
export declare enum DialogVariant {
    DEFAULT = 0,
    INFO = 1,
    SUCCESS = 2,
    WARNING = 3,
    ERROR = 4
}
export type DialogButtonProps = TestProps & {
    /** Text to show on the dialog button. */
    text: string;
    /** The button variant to show. */
    variant?: ButtonVariant;
    /** If `true`, disable all interactions for this button. */
    disabled?: boolean;
    /** Callback to invoke on dialog button press. */
    onPress(): void;
};
export type DialogProps = TestProps & {
    /** Display the dialog only when value is `true`. Defaults to `false`.*/
    visible?: boolean;
    /** Text to show in the header of the dialog. If not present, the dialog header won't be visible. */
    title?: string;
    /** Larger text to show above the content text as the headline of the dialog content. */
    headline?: string;
    /**
     * Icon component to render to the left of the title.
     * Overrides icon coming from `variant`.
     */
    icon?: JSX.Element;
    /**
     * Custom highlight color to use for the header.
     * Overrides color coming from `variant`.
     */
    highlightColor?: Colors;
    /**
     * Text to display as the body of the dialog. If not present, children will be rendered instead.
     */
    description?: string;
    /**
     * Smaller text that is displayed at the bottom of the dialog, above buttons.
     */
    footerText?: string;
    /** First button of the modal. */
    firstButton?: DialogButtonProps;
    /** Second button of the modal. Use only when `firstButton` is also present. */
    secondButton?: DialogButtonProps;
    /** If `true`, don't stretch buttons to fill entire width. Defaults to `false`. */
    smallButtons?: boolean;
    /** If `true`, the buttons will be aligned vertically. Defaults to `false`. */
    verticalButtons?: boolean;
    /**
     * The dialog type to show. Affects highlight colors and header icon.
     *
     * Defaults to `DialogVariant.DEFAULT`
     */
    variant?: DialogVariant;
    /**
     * If `true`, renders the dialog inline, skipping the underlying Modal component from being rendered. `visible` property has no effect when dialog is inline.
     *
     * Defaults to `false`.
     *
     * NOTE: Use with caution, this is a hack to circumvent `Modal` issues on iOS in specific cases.
     */
    inline?: boolean;
    /**
     * Callback to invoke when close icon is pressed or closing is requested
     * (e.g. back button pressed on android).
     *
     * Close icon is only shown if callback is provided and the title property is defined.
     */
    onClose?: () => void;
    /** Custom styling to apply to the dialog body. */
    style?: StyleProp<ViewStyle>;
    /**
     * Components to display.
     */
    children?: ReactNode;
    /**
     * If `true`, wraps the Dialog content into a KeyboardContainer that fixes the keyboard covering the bottom of the Dialog issue on iOS.
     *
     * Defaults to `false`.
     *
     * Enable it if the Dialog contains inputs.
     *
     * NOTE: This has no effect if the `inline` property is `true`.
     */
    withInput?: boolean;
};
export declare const Dialog: FC<DialogProps>;
//# sourceMappingURL=dialog.d.ts.map