import { Colors } from "./colors";
import { FontFaces, FontSize } from "./fonts";
import { IconSize } from "../components/icons/icon-size";
export type ListItemStyle = {
    fillColor: Colors;
    lineColor: Colors;
};
type SegmentedControlColors = {
    /** Color of tab's text and icon fill. */
    readonly contentColor: Colors;
    /** Color of tab's background. */
    readonly backgroundColor: Colors;
    /** Color of tab's border. */
    readonly borderColor: Colors;
};
type NativeSwitchStyle = {
    /** Color of switch background. */
    readonly trackColor: Colors;
    /** Color of switch thumb. */
    readonly thumbColor: Colors;
    /**
     * Color of border surrounding switch component.
     *
     * _Works on iOS only._
     */
    readonly borderColor: Colors;
};
/** Styles for checkboxes and radio buttons. */
export type ToggleControlStyle = {
    readonly borderColor: Colors;
    readonly fillColor: Colors;
};
type ToggleControlStates = {
    readonly default: ToggleControlStyle;
    readonly pressed: ToggleControlStyle;
    readonly disabled: ToggleControlStyle;
};
/** Styles for switches. */
export type SwitchStyle = {
    readonly borderColor: Colors;
    readonly fillColor: Colors;
    readonly centerColor: Colors;
};
type SwitchStates = {
    readonly default: SwitchStyle;
    readonly pressed: SwitchStyle;
    readonly disabled: SwitchStyle;
};
type SliderStyle = {
    readonly backgroundColor: Colors;
    readonly color: Colors;
};
export type SignalStyle = {
    readonly fillColor: Colors;
    readonly lineColor: Colors;
    readonly textColor: Colors;
};
export interface Theme {
    /** Color of text component. */
    readonly textColor: Colors;
    /** Text color to use for primary buttons or on differently colored backgrounds. */
    readonly secondaryTextColor: Colors;
    /** Background color of the screens. */
    readonly backgroundColor: Colors;
    /** Background color of the secondary screens. */
    readonly secondaryBackgroundColor: Colors;
    /** Color of primary buttons, process indicators. */
    readonly primaryColor: Colors;
    /** Pressed color version of `primaryColor`. */
    readonly pressedColor: Colors;
    /** Pressed color used for secondary and ternary buttons. */
    readonly secondaryPressedColor: Colors;
    /** Color to use for disabled component backgrounds, borders, labels etc. */
    readonly disabledColor: Colors;
    /** Color to use for disabled component texts when background is also set. */
    readonly disabledTextColor: Colors;
    /** Name of regular font to use. */
    readonly regularFont: FontFaces;
    /** Name of bold font to use. */
    readonly boldFont: FontFaces;
    /**
     * Specifies whether fonts should scale to respect Text Size accessibility settings.
     *
     * WARNING: setting this option to `false` is bad for accessibility, only do so when
     * it is a hard requirement!
     */
    readonly allowFontScaling: boolean;
    /** Whether the theme is a dark or a light theme. */
    readonly dark: boolean;
    /** Status bar style. Controls color of text and icons. */
    readonly statusBarStyle: "light-content" | "dark-content";
    /** Keyboard appearance for the text fields on iOS. */
    readonly keyboardAppearance: "light" | "dark";
    /** Whether to render loading indicator with colors. */
    readonly coloredLoadingIndicator: boolean;
    /** Whether or not to show the Bosch supergraphic, e.g. below the header. */
    readonly showSupergraphic: boolean;
    /** Color for horizontal, vertical lines and list separators. */
    readonly separatorColor: Colors;
    /** Width of separator line. */
    readonly separatorSize: number;
    /** Background color of modal. */
    readonly modalBackgroundColor: Colors;
    /** Color for hint labels, like text field counter or info labels. */
    readonly hintLabelColor: Colors;
    /** Color of list section header background. */
    readonly sectionBackgroundColor: Colors;
    /** Color of list item description texts. */
    readonly listSecondaryColor: Colors;
    /** Text color inside the tiles. */
    readonly tileTextColor: Colors;
    /** Opacity to use for TouchableOpacity. When pressed, the items within will have their opacity set to this value. */
    readonly activeOpacity: number;
    /** Colors for error-related UI elements. */
    readonly error: SignalStyle;
    /** Colors for warning related UI elements. */
    readonly warning: SignalStyle;
    /** Colors for the correct UI elements. */
    readonly success: SignalStyle;
    /** Colors for information elements. */
    readonly info: SignalStyle;
    readonly checkbox: {
        readonly borderWidth: number;
        /** Size of the shown icon. */
        readonly iconSize: number;
        /** Color of selected checkbox icon - colored checkbox. */
        readonly coloredTickColor: Colors;
        /** Color of selected checkbox icon - simple checkbox. */
        readonly tickColor: Colors;
        readonly disabledTickColor: Colors;
        readonly checked: ToggleControlStates;
        readonly unchecked: ToggleControlStates;
    };
    readonly inputs: {
        /** Color of the placeholder text when not disabled. */
        readonly placeholderColor: Colors;
        /**
         * The color of the selection.
         * iOS automatically applies some opacity to the passed color,
         * but on Android care must be taken so that the color is not too strong
         * for the text.
         */
        readonly selectionColor: {
            readonly ios: Colors;
            readonly android: Colors;
        };
        readonly default: SignalStyle;
        readonly focused: SignalStyle;
        readonly pressed: SignalStyle;
        readonly disabled: SignalStyle;
    };
    readonly list: {
        readonly default: ListItemStyle;
        readonly pressed: ListItemStyle;
        readonly disabled: ListItemStyle;
    };
    readonly nativeSwitch: {
        readonly enabledOn: NativeSwitchStyle;
        readonly ios: {
            readonly disabled: NativeSwitchStyle;
            readonly enabledOff: NativeSwitchStyle;
        };
        readonly android: {
            readonly disabled: NativeSwitchStyle;
            readonly enabledOff: NativeSwitchStyle;
        };
    };
    readonly progressBar: {
        /** Color to use progress bar tracks. */
        readonly trackColor: Colors;
    };
    readonly radioButton: {
        readonly borderWidth: number;
        readonly centerColor: Colors;
        readonly pressedCenterColor: Colors;
        readonly disabledCenterColor: Colors;
        readonly selected: ToggleControlStates;
        readonly unselected: ToggleControlStates;
    };
    readonly segmentedControl: {
        /** The fontsize of the segmented control tab's labels. */
        readonly fontSize: FontSize;
        /** The icon size of the segmented control tab's icon. */
        readonly iconSize: IconSize;
        readonly selected: {
            readonly regular: SegmentedControlColors;
            readonly disabled: SegmentedControlColors;
            readonly pressed: SegmentedControlColors;
        };
        readonly tab: {
            readonly regular: SegmentedControlColors;
            readonly disabled: SegmentedControlColors;
            readonly pressed: SegmentedControlColors;
        };
    };
    readonly slider: {
        readonly default: SliderStyle;
        readonly pressed: SliderStyle;
        readonly disabled: SliderStyle;
    };
    readonly switch: {
        readonly borderWidth: number;
        readonly on: SwitchStates;
        readonly off: SwitchStates;
    };
    readonly whatsNew: {
        /** Background of What's new modal. */
        readonly backgroundColor: Colors;
        /** Text color of What's new modal. */
        readonly textColor: Colors;
        /** Color of button when pressed. */
        readonly pressedColor: Colors;
        /** Background color of button when pressed. */
        readonly secondaryPressedColor: Colors;
    };
}
export declare const boschTheme: Theme;
export declare const buderusTheme: Theme;
export {};
//# sourceMappingURL=themes.d.ts.map