package com.bosch.tt.de.emma.connector;

import android.os.Build;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.ProcessLifecycleOwner;

import com.bosch.sh.connector.thirdparty.ThirdPartyConnectorLibraryImpl;
import com.bosch.sh.connector.thirdparty.api.ThirdPartyConnectorLibrary;
import com.bosch.sh.connector.thirdparty.api.http.Result;
import com.bosch.sh.connector.thirdparty.api.pairing.PairingListener;
import com.bosch.sh.connector.thirdparty.api.pairing.PairingResult;
import com.bosch.sh.connector.thirdparty.api.pairing.PairingStatus;
import com.bosch.sh.connector.thirdparty.api.pairing.PairingStatusListener;
import com.bosch.sh.connector.thirdparty.api.shc.ShcIdChangeResult;
import com.bosch.sh.connector.thirdparty.api.shc.ShcIdChangeResultListener;
import com.bosch.tt.de.emma.connector.longpolling.LongPollingListener;
import com.bosch.tt.de.emma.connector.longpolling.LongPollingService;
import com.bosch.tt.de.emma.connector.service.RequestService;
import com.bosch.tt.de.emma.connector.service.ShcCallback;
import com.bosch.tt.de.emma.connector.utils.JsonConverter;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.UiThreadUtil;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;
import java.util.concurrent.RunnableFuture;

public class RNShcConnectorModule extends ReactContextBaseJavaModule implements LongPollingListener {

	private static final String TAG = "RNShcConnector";

	private static final String RNSHC_CONNECTED = "RNSHC_CONNECTED";
	private static final String RNSHC_DISCONNECTED = "RNSHC_DISCONNECTED";

	private static final String RNSHC_ID_CHANGE_RESULT = "RNSHC_ID_CHANGE_RESULT";
	private static final String RNSHC_PARING_STATUS_CHANGED = "RNSHC_PARING_STATUS_CHANGED";
	private static final String RNSHC_PARING_STATUS = "RNSHC_PARING_STATUS";

	private final ReactApplicationContext reactContext;
	private final LongPollingService longPollingService;

	/**
	 * The controller for the SHC connection.
	 */
	private ThirdPartyConnectorLibrary thirdPartyConnectorLibrary;

	public RNShcConnectorModule(ReactApplicationContext reactContext) {
		super(reactContext);
		this.reactContext = reactContext;
		this.longPollingService = LongPollingService.getInstance();
	}

	@Override
	@NonNull
	public String getName() {
		return TAG;
	}

	private synchronized ThirdPartyConnectorLibrary getThirdPartyConnectorLibrary() {
		if (thirdPartyConnectorLibrary == null) {
			Log.v(TAG, "Creating connection controller.");

			Runnable initConnLib = new Runnable() {
				@Override
				public void run() {
					LifecycleOwner lifecycleOwner = ProcessLifecycleOwner.get();
					thirdPartyConnectorLibrary = ThirdPartyConnectorLibraryImpl.init(
						RNShcConnectorModule.this.reactContext,
						lifecycleOwner
					);
				}
			};
			RunnableFuture<Void> task = new FutureTask<>(initConnLib, null);

			// Run init on UI thread, since lifecycle observers require being added on the main thread 
			UiThreadUtil.runOnUiThread(task);
			try {
				task.get(); // this will block until Runnable completes
			} catch (InterruptedException | ExecutionException e) {
				// If exception occurred while executing runnable, do nothing
				// since `thirdPartyConnectorLibrary` will just be `null`.
			}
		}

		return thirdPartyConnectorLibrary;
	}

	@ReactMethod
	public void initModule() {
		getThirdPartyConnectorLibrary();
	}

	@ReactMethod
	public void connect() {
		Log.v(TAG, "connect");

		longPollingService.subscribe(this, getThirdPartyConnectorLibrary());
	}


	@ReactMethod
	public void hasShcIdBeenProvided(Promise promise) {
		Log.v(TAG, "hasShcIdBeenProvided");
		if (getThirdPartyConnectorLibrary().hasShcIdBeenProvided()) {
			promise.resolve("The SHC ID has already provided");
		} else {
			promise.reject("SHC_ID_NOT_SET");
		}
	}

	@ReactMethod
	public void setShcId(final @NonNull String shcId) {
		Log.v(TAG, "setShcId " + shcId);

		getThirdPartyConnectorLibrary().setShcId(shcId, new ShcIdChangeResultListener() {
			@Override
			public void onShcIdSet(ShcIdChangeResult shcIdChangeResult) {
				RNShcConnectorModule.this.sendShcIdChangedEvent(shcIdChangeResult);
			}
		});
	}

	@ReactMethod
	public void setShcIdAndLocalAddress(final @NonNull String shcId, final @NonNull String address, final Promise promise) {
		Log.v(TAG, "setShcIdAndLocalAddress: " + shcId + ", " + address);

		getThirdPartyConnectorLibrary().setShcId(shcId, new ShcIdChangeResultListener() {
			@Override
			public void onShcIdSet(ShcIdChangeResult shcIdChangeResult) {
				RNShcConnectorModule.this.sendShcIdChangedEvent(shcIdChangeResult);
				getThirdPartyConnectorLibrary().setLocalIp(address);
				promise.resolve(null);
			}
		});

	}

	@ReactMethod
	public void checkPairStatus() {
		Log.v(TAG, "Get paired Status ");
		thirdPartyConnectorLibrary.checkPairingStatus(new PairingStatusListener() {
			@Override
			public void receivedPairingStatus(PairingStatus pairingStatus) {
				Log.v(TAG, "Check pair Status " + pairingStatus.name());
				WritableMap params = Arguments.createMap();
				params.putString("status", pairingStatus.name());
				sendEvent(RNSHC_PARING_STATUS, params);
			}
		});
	}

	@ReactMethod
	public void pair(final String id, final String nonce) {
		thirdPartyConnectorLibrary.pair(id, nonce, "EM_" + Build.MODEL, new PairingListener() {
			@Override
			public void pairingFinished(PairingResult pairingResult) {
				Log.v(TAG, "Pair Status Event " + pairingResult.name());
				WritableMap params = Arguments.createMap();
				params.putString("status", pairingResult.name());
				sendEvent(RNSHC_PARING_STATUS_CHANGED, params);
			}
		});
	}

	@ReactMethod
	public void get(final String url, final Callback successCallback, final Callback errorCallback) {
		RequestService.get(url, getThirdPartyConnectorLibrary(), new ShcCallback() {
			@Override
			public void onResult(Result result) {
				handleResponse(result, successCallback, errorCallback);
			}
		});
	}

	@ReactMethod
	public void put(final String url, final String body, final Callback successCallback, final Callback errorCallback) {
		RequestService.put(url, body, getThirdPartyConnectorLibrary(), new ShcCallback() {
			@Override
			public void onResult(Result result) {
				handleResponse(result, successCallback, errorCallback);
			}
		});
	}

	@ReactMethod
	public void patch(final String url, final String body, final Callback successCallback, final Callback errorCallback) {
		RequestService.patch(url, body, getThirdPartyConnectorLibrary(), new ShcCallback() {
			@Override
			public void onResult(Result result) {
				handleResponse(result, successCallback, errorCallback);
			}
		});
	}


	@ReactMethod
	public void addListener(String eventName) {
		// We don't have any upstream listeners or background tasks so we can leave it empty
	}
	
	@ReactMethod
	public void removeListeners(Integer count) {
		// We don't have any upstream listeners or background tasks so we can leave it empty
	}

	@Override
	public void connectionLost() {
		sendEvent(RNSHC_DISCONNECTED, null);
	}

	@Override
	public void connected() {
		sendEvent(RNSHC_CONNECTED, null);
	}

	private void sendEvent(String eventName, @Nullable WritableMap params) {
		this
			.reactContext
			.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
			.emit(eventName, params);
	}

	private void handleResponse(final Result result, final Callback successCallback, final Callback errorCallback) {
		// If getCause returns a non-null value, it means that an internal error happened
		if (result.getCause() != null) {
			errorCallback.invoke(createErrorMessage(result.getCode(), result.getMessage()));
			return;
		}

		WritableMap params = Arguments.createMap();
		// If no internal error has occurred the result.getCode() returns the HTTP result code
		int httpResultCode = result.getCode();
		params.putInt("code", httpResultCode);

		if (!result.getMessage().isEmpty()) {
			try {
				Object json = new JSONTokener(result.getMessage()).nextValue();
				if (json instanceof JSONObject) {
					params.putMap("body", JsonConverter.convertJsonToMap(new JSONObject(result.getMessage())));
				} else if (json instanceof JSONArray) {
					params.putArray("body", JsonConverter.convertJsonToArray(new JSONArray(result.getMessage())));
				} else {
					params.putString("body", json.toString());
				}
			} catch (JSONException e) {
				errorCallback.invoke(createErrorMessage(httpResultCode, e.getMessage()));
				return;
			}
		}
		if ((httpResultCode >= 200) && (httpResultCode < 300)) {
			successCallback.invoke(params);
		} else {
			errorCallback.invoke(params);
		}
	}

	private WritableMap createErrorMessage(final int errorCode, final String message) {
		String convertedErrorCode = convertConLibError(errorCode);
		WritableMap error = Arguments.createMap();

		if (convertedErrorCode == null) {
			error.putInt("code", errorCode);
		} else {
			error.putString("code", convertedErrorCode);
		}
		error.putString("message", message);

		return error;
	}

	private void sendShcIdChangedEvent(final ShcIdChangeResult shcIdChangeResult) {
		Log.v(TAG, "SHC ID changed: " + shcIdChangeResult.name());
		WritableMap params = Arguments.createMap();
		params.putString("status", shcIdChangeResult.name());
		sendEvent(RNSHC_ID_CHANGE_RESULT, params);
	}

	/**
	 * Converts the internal error code to a more descriptive one.
	 * These codes are coming from the connector library.
	 * Please see: {@link com.bosch.sh.connector.thirdparty.api.http.CommonResults}
	 *
	 * @param code error code which comes during a failed request
	 * @return more descriptive name about the error code
	 */
	private String convertConLibError(final int code) {
		switch (code) {
			case 1:
				return "SSL_HANDSHAKE_FAILURE";
			case 2:
				return "NO_ENDPOINT_AVAILABLE";
			case 3:
				return "BACKEND_UNREACHABLE";
			case 4:
				return "REQUEST_QUEUE_FULL";
			case 5:
				return "REQUEST_CANCELLED";
			case 6:
				return "REQUEST_TIMED_OUT";
			case 7:
				return "SHC_ID_NOT_SET";
			case 8:
				return "SHC_ID_DOES_NOT_MATCH";
			case 9:
				return "UNKNOWN_ERROR";
			default:
				return null;
		}
	}
}
