package com.bosch.tt.de.emma.connector.longpolling;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import androidx.annotation.NonNull;

import com.bosch.sh.connector.thirdparty.api.ThirdPartyConnectorLibrary;
import com.bosch.sh.connector.thirdparty.api.http.HttpResponseHandler;
import com.bosch.sh.connector.thirdparty.api.http.Result;
import com.bosch.tt.de.emma.connector.service.RequestService;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;

import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * LongPollingService is responsible for detecting if the state of the connection  between the application and the Smart Home Controller changed.
 */
public class LongPollingService {
	private static final String TAG = LongPollingService.class.getSimpleName();
	private static final String LONG_POLLING_URL = "/remote/json-rpc";
	private static final MediaType JSON = MediaType.parse("application/json; charset=utf-8");
	private static final int INTERVAL_MS = 20000;

	private static LongPollingService INSTANCE = null;

	private LongPollingListener listener;
	private Handler mHandler;
	private String longPollingId;
	private ThirdPartyConnectorLibrary thirdPartyConnectorLibrary;
	private boolean isConnected;
	private RequestBody pollRequestBody = null;

	/**
	 * Calls the poll request after every 7 seconds. Starts running after a successful subscribe call.
	 */
	Runnable connectionStateChecker = new Runnable() {
		@Override
		public void run() {
			try {
				poll();
			} finally {
				mHandler.postDelayed(connectionStateChecker, INTERVAL_MS);
			}
		}
	};

	private LongPollingService() {
		this.listener = null;
	}

	public static LongPollingService getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new LongPollingService();
		}

		return INSTANCE;
	}

	/**
	 * Subscribes to `/remote/json-rpc` request on the Smart Home Controller side.
	 * If the subscribe request returns with success, it starts the periodically polling the endpoint to detect connection changes.
	 *
	 * @param listener                   listener to be called when a change occurs
	 * @param thirdPartyConnectorLibrary an instance of the connector library used for sending http requests to Smart Home Controller
	 */
	public void subscribe(@NonNull final LongPollingListener listener, @NonNull final ThirdPartyConnectorLibrary thirdPartyConnectorLibrary) {
		this.listener = listener;
		RequestBody requestBody = createRequestBody("RE/subscribe", "com/bosch/sh/remote/*", "null");

		this.thirdPartyConnectorLibrary = thirdPartyConnectorLibrary;

		final Request.Builder requestBuilder = new Request.Builder().post(requestBody);

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(LONG_POLLING_URL), requestBuilder, new HttpResponseHandler() {
			@Override
			public void onResponse(Response response) {
				try {
					Log.v(TAG, "Successfully subscribed to LongPolling");
					longPollingId = getLongPollingId(response.body().string());
					mHandler = new Handler(Looper.getMainLooper());
					connectionStateChecker.run();
				} catch (IOException | JSONException e) {
					Log.e(TAG, "The response does not contain the long polling id", e.getCause());
				}
			}

			@Override
			public void onFailure(Result result) {
				Log.e(TAG, "Error in subscribe to LongPolling");
			}
		});
	}

	/**
	 * Stops sending notifications about the connection change and calls the unsubscribe request from the Smart Home Controller.
	 * NOTE: Currently the EMMA application cannot go into a disconnected state by user interaction, so this function is not called anywhere.
	 */
	public void unsubscribe() {
		RequestBody requestBody = createRequestBody("RE/unsubscribe", longPollingId);

		final Request.Builder requestBuilder = new Request.Builder().post(requestBody);
		listener = null;

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(LONG_POLLING_URL), requestBuilder, new HttpResponseHandler() {
			@Override
			public void onResponse(Response response) {
				Log.i(TAG, "Successfully unsubscribe from LongPolling");
				mHandler.removeCallbacks(connectionStateChecker);
				pollRequestBody = null;
			}

			@Override
			public void onFailure(Result result) {
				Log.e(TAG, "Error in unsubscribe from LongPolling");
			}
		});
	}

	/**
	 * Sends a poll request to the Smart Home Controller.
	 * It notifies the listener whenever the outcome (success/failure) changes
	 * compared to the previous request.
	 */
	private void poll() {
		if (pollRequestBody == null) {
			pollRequestBody = createRequestBody("RE/longPol", "2");
		}

		final Request.Builder requestBuilder = new Request.Builder().post(pollRequestBody);

		thirdPartyConnectorLibrary.httpClient().executeAsync(RequestService.createUrlBuilder(LONG_POLLING_URL), requestBuilder, new HttpResponseHandler() {
			@Override
			public void onResponse(Response response) {
				Log.v(TAG, "Poll function successfully returned");

				// Notify listener of connection established if previous state was disconnected 
				if (!isConnected) {
					isConnected = true;
					listener.connected();
				}
			}

			@Override
			public void onFailure(Result result) {
				// Notify listener of connection lost if previous state was connected 
				if (isConnected) {
					isConnected = false;
					listener.connectionLost();
				}
			}
		});
	}

	/**
	 * Extracts the subscription id from the response JSON.
	 */
	private String getLongPollingId(String json) throws JSONException {
		String id = "";
		JSONArray array = new JSONArray(json);
		if (array.length() > 0) {
			id = array.getJSONObject(0).getString("result");
		}

		return id;
	}

	private RequestBody createRequestBody(@NonNull final String method, @NonNull final String... paramList) {
		try {
			JSONObject bodyObject = new JSONObject();
			JSONArray params = new JSONArray();
			
			for (String param : paramList) {
				params.put(param);
			}

			// The fields of the request body come from Smart Home team, so we don't know the meaning of the `randomid` or `jsonrpc: 2.0`
			bodyObject.put("jsonrpc", "2.0");
			bodyObject.put("method", method);
			bodyObject.put("id", "randomid");
			bodyObject.put("params", params);

			JSONArray bodyArray = new JSONArray();
			bodyArray.put(bodyObject);

			return RequestBody.create(bodyArray.toString(), JSON);

		} catch (JSONException e) {
			Log.e(TAG, "Creating the long polling request body has failed", e.getCause());
			return null;
		}
	}
}
