/**
 * bosch-tt-react-native-shc-connector module
 */
declare module "bosch-tt-react-native-shc-connector" {
	import { NativeEventEmitter, EmitterSubscription } from "react-native";

	export interface ICallback {
		(...args: any[]): void;
	}

	/**
	 * Predefined events which are supported by the RNConnector module
	 * RNSHC_ID_CHANGE_RESULT - Android specific, it is triggered when the SHC ID is changed
	 * RNSHC_PARING_STATUS_CHANGED - Android specific, it is triggered when the Pairing Status is changed (after the pair function)
	 * RNSHC_PARING_STATUS- Android specific, it is triggered when the Pairing Status is changed (after the check pair state function)
	 * RNSHC_CONNECTION_LOST
	 * RNSHC_CONNECTED
	 * RNSHC_DISCONNECTED
	 */
	export enum RNShcEventType {
		ShcIdChangedResult = "RNSHC_ID_CHANGE_RESULT",
		PairingStatusChanged = "RNSHC_PARING_STATUS_CHANGED",
		PairingStatus = "RNSHC_PARING_STATUS",
		ConnectionLost = "RNSHC_CONNECTION_LOST",
		Connected = "RNSHC_CONNECTED",
		Disconnected = "RNSHC_DISCONNECTED",
	}

	/**
	 * RNShcConnector interface
	 */
	export default class RNShcConnector {
		private emitter: NativeEventEmitter;
		/**
		 * Registers an event listener for the eventType which is specified in the parameter list
		 * @param eventType predefined event, please see the description of the RNShcEventType
		 * @param listener function to invoke when the specified event is emitted
		 * @param context optional object to use when invoking the listener
		 */
		static addEventListener<T>(
			eventType: RNShcEventType,
			listener: (arg: T) => any,
			context?: any,
		): EmitterSubscription;

		/**
		 * Remove a handler by passing the RNShcEventType
		 */
		static removeEventListener(eventType: RNShcEventType): void;

		/**
		 * Initializes the connector library
		 */
		static initModule(): void;

		/**
		 * Android specific function
		 * Connects to the Smart Home Controller and register the connection change listeners
		 */
		static connect(): void;

		/**
		 * Checks if the SHC id is provided previously
		 * Returns a promise which resolves if the SHC ID is already provided and
		 * reject with SHC_ID_NOT_SET code if the SHC ID is not configured
		 * only on iOS side: reject with CONNECTION_ERROR code if the configured SHC is not available
		 */
		static hasShcIdBeenProvided<T>(): Promise<T>;

		/**
		 * Android specific function
		 * Sets up the SHC ID or MAC address
		 * @param id the MAC address of the Smart Home Controller. Example: 65-da-02-0a-43-9c
		 * @event RNSHC_ID_CHANGE_RESULT is triggerd if the setting the SHC ID is ready
		 */
		static setShcId<T>(id: string): Promise<T>;

		/**
		 * Set up the Smart Home Conroller ID and the local IP address
		 * @param id the MAC address of the Smart Home Controller. Example: 65-da-02-0a-43-9c
		 * @param address local IP address of the Smart Home Controller. Example: 10.10.66.80
		 * @returns Promise object, which in case of iOS resolves if the credentials are valid, rejects otherwise.
		 * Always resolves on Android after the mdns lookup returns, but have to call the `checkPairStatus` to verify the result.
		 */
		static setShcIdAndLocalAddress<T>(
			id: string,
			address: string,
		): Promise<T>;

		/**
		 * Checks the pair status
		 * In android triggers a RNSHC_PARING_STATUS event
		 * In iOS returns a Promise object, with the proper pairing status
		 * (Possible values: ShcConRepresentationNotConfigured, ShcConRepresentationNotPaired,
		 * ShcConRepresentationPaired, ShcConRepresentationOutOfDate, ShcConnectivityCompatible,
		 * ShcConnectivitySHCOutdated, ShcConnectivityClientOutdated)
		 *
		 */
		static checkPairStatus<T>(): Promise<T>;

		/**
		 * Pairs the EMMA application to the Smart Home Controller
		 * @param id identifaction of the pairing workflow
		 * @param nonce authentication for the pairing of the client
		 * In android triggers a RNSHC_PARING_STATUS_CHANGED event
		 * In iOS returns an Promise object
		 */
		static pair<T>(id: string, nonce: string): Promise<T>;

		/**
		 * Create a HTTP GET request
		 * @param url contains the EMMA specific part of the URL. Example: smarthome-ext/emma/state
		 * @param successCallback callback function which will be called when the request succeeded
		 * @param errorCallback callback function which will be called when the request failed
		 */
		static get<T>(
			url: string,
			successCallback: (result: T) => void,
			errorCallback: (error: any) => void,
		): void;

		/**
		 * Create a HTTP PUT request
		 * @param url contains the EMMA specific part of the URL. Example: smarthome-ext/emma/state
		 * @param body json object
		 * @param successCallback callback function which will be called when the request succeeded
		 * @param errorCallback callback function which will be called when the request failed
		 */
		static put<T>(
			url: string,
			body: string,
			successCallback: (result: T) => void,
			errorCallback: (error: any) => void,
		): void;

		/**
		 * Create a HTTP PATCH request
		 * @param url contains the EMMA specific part of the URL. For ex. smarthome-ext/emma/state
		 * @param body json object
		 * @param successCallback callback function which will be called when the request succeeded
		 * @param errorCallback callback function which will be called when the request failed
		 */
		static patch<T>(
			url: string,
			body: string,
			successCallback: (result: T) => void,
			errorCallback: (error: any) => void,
		): void;

		/**
		 * Clear the pair data from the application memory (Ex.: Smart Home Controller ID or IP address)
		 * This function is only available on the iOS side.
		 */
		static clearPairData(): void;

		/**
		 * iOS function
		 * Refresh the connection type (REMOTE/LOCALE)
		 * This function call is necessary on iOS side, when the application
		 * is moving out of the background to back into the foreground
		 */
		static refreshConnection(): void;
	}
}
