package com.bosch.tt.privacypolicy.rn;

import androidx.annotation.NonNull;
import android.util.Log;
import android.util.SparseArray;

import com.bosch.tt.privacypolicy.core.ActivityDocument;
import com.bosch.tt.privacypolicy.core.Country;
import com.bosch.tt.privacypolicy.core.Document;
import com.bosch.tt.privacypolicy.core.MetaActivityDocument;
import com.bosch.tt.privacypolicy.core.MetaDocument;
import com.bosch.tt.privacypolicy.core.TTActivityProvider;
import com.bosch.tt.privacypolicy.core.TTActivityProviderBuilder;
import com.bosch.tt.privacypolicy.core.TTPrivacyProvider;
import com.bosch.tt.privacypolicy.core.TTPrivacyProviderBuilder;
import com.bosch.tt.privacypolicy.rn.assembler.ActivityConsentAssembler;
import com.bosch.tt.privacypolicy.rn.provider.ActivityConsentOfflineProvider;
import com.bosch.tt.privacypolicy.rn.provider.PrivacyPolicyOfflineProvider;
import com.bosch.tt.privacypolicy.rn.provider.TermsOfUseProvider;
import com.bosch.tt.privacypolicy.ui.CountrySelector;
import com.bosch.tt.privacypolicy.ui.TTActivityComponent;
import com.bosch.tt.privacypolicy.ui.TTPrivacyComponent;
import com.bosch.tt.privacypolicy.valueprovider.PreviewValueProvider;
import com.bosch.tt.privacypolicy.valueprovider.ProductionValueProvider;
import com.bosch.tt.privacypolicy.valueprovider.ValueProvider;
import com.bosch.tt.privacypolicy.rn.assembler.CountryAssembler;
import com.bosch.tt.privacypolicy.rn.assembler.DocumentAssembler;
import com.bosch.tt.privacypolicy.rn.ui.TTComponentEvent;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.uimanager.IllegalViewOperationException;
import com.facebook.react.uimanager.NativeViewHierarchyManager;
import com.facebook.react.uimanager.UIBlock;
import com.facebook.react.uimanager.UIManagerModule;
import com.facebook.react.uimanager.events.RCTEventEmitter;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class RNPrivacyComponentModule extends ReactContextBaseJavaModule {
    private static final String TAG = RNPrivacyComponentModule.class.getSimpleName();
    private static final DateFormat isoDateFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

    private final ReactApplicationContext reactContext;

    private TTPrivacyProvider privacyProvider;
    private TTPrivacyProvider termsOfUseProvider;
    private TTActivityProvider activityProvider;

    private CountryAssembler countryAssembler;
    private DocumentAssembler documentAssembler;
    private ActivityConsentAssembler activityConsentAssembler;

    private List<Country> supportedCountries;
    private Country selectedCountry;
    private CountrySelector.CountrySelectedListener countrySelectedListener;

    /**
     * Every TTPrivacyComponent has its own listener identified by its id
     */
    private SparseArray<TTPrivacyComponent.OnLoadFinishedListener> onDocumentLoadListeners;
    /**
     * Every TTActivityComponent has its own listener identified by its id
     */
    private SparseArray<TTActivityComponent.OnLoadFinishedListener> onActivityDocumentLoadListeners;

    public RNPrivacyComponentModule(final ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
        this.onDocumentLoadListeners = new SparseArray<>();
        this.countryAssembler = new CountryAssembler(reactContext);
        this.documentAssembler = new DocumentAssembler();
        this.countrySelectedListener = new CountrySelector.CountrySelectedListener() {
            @Override
            public void onCountrySelected(CountrySelector countrySelector, Country country) {
                countrySelector.setSelectedCountry(country);
                reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(countrySelector.getId(), TTComponentEvent.ON_COUNTRY_SELECTED.toString(), countryAssembler.createWritableDTO(country));
            }
        };
    }

    @Override
    @NonNull
    public String getName() {
        return "RNPrivacyComponent";
    }


    // ---------------------------------------------- Country related methods ----------------------------------------------

    @ReactMethod
    public void loadCountries(final int viewTag) {
        final UIManagerModule uiManager = this.reactContext.getNativeModule(UIManagerModule.class);
        uiManager.addUIBlock(new UIBlock() {
            @Override
            public void execute(NativeViewHierarchyManager nativeViewHierarchyManager) {
                try {
                    final CountrySelector countrySelector = (CountrySelector) nativeViewHierarchyManager.resolveView(viewTag);
                    countrySelector.loadCountries(privacyProvider, supportedCountries, selectedCountry, countrySelectedListener);
                    Log.d(TAG, "loadCountries method got called");
                } catch (IllegalViewOperationException e) {
                    Log.e(TAG, e.getMessage());
                }
            }
        });
    }

    @ReactMethod
    public void setSupportedCountries(final ReadableArray countries, final int viewTag) {
        this.supportedCountries = countryAssembler.createModelArray(countries);
        final UIManagerModule uiManager = this.reactContext.getNativeModule(UIManagerModule.class);
        uiManager.addUIBlock(new UIBlock() {
            @Override
            public void execute(NativeViewHierarchyManager nativeViewHierarchyManager) {
                try {
                    final CountrySelector countrySelector = (CountrySelector) nativeViewHierarchyManager.resolveView(viewTag);
                    countrySelector.setCountrySelectedListener(countrySelectedListener);
                    countrySelector.setCountries(supportedCountries);
                    Log.d(TAG, "setSupportedCountries method got called with " + supportedCountries.size() + " countries.");
                } catch (IllegalViewOperationException e) {
                    Log.e(TAG, e.getMessage());
                }
            }
        });
    }

    @ReactMethod
    public void setSelectedCountry(final ReadableMap country, final int viewTag) {
        this.selectedCountry = countryAssembler.createModel(country);
        final UIManagerModule uiManager = this.reactContext.getNativeModule(UIManagerModule.class);
        uiManager.addUIBlock(new UIBlock() {
            @Override
            public void execute(NativeViewHierarchyManager nativeViewHierarchyManager) {
                try {
                    final CountrySelector countrySelector = (CountrySelector) nativeViewHierarchyManager.resolveView(viewTag);
                    countrySelector.setSelectedCountry(selectedCountry);
                    Log.d(TAG, "setSelectedCountry method got called with " + selectedCountry.toString());
                } catch (IllegalViewOperationException e) {
                    Log.e(TAG, e.getMessage());
                }
            }
        });
    }


    // ---------------------------------------------- Privacy Policy related methods ----------------------------------------------


    @ReactMethod
    public void initPrivacyProvider(final String leanIX, final boolean productionMode) {
        final ValueProvider onlineProvider = productionMode ? new ProductionValueProvider(leanIX) : new PreviewValueProvider(leanIX);
        this.privacyProvider = new TTPrivacyProviderBuilder(this.reactContext, leanIX, new PrivacyPolicyOfflineProvider(this.reactContext)).onlineProvider(onlineProvider).build();
    }

    @ReactMethod
    public void loadPrivacyPolicy(final ReadableMap country, final int viewTag) {
        this.loadDocumentWithProvider(this.privacyProvider, country, viewTag);
    }

    @ReactMethod
    public void acceptPrivacyPolicy(final String documentId) {
        this.privacyProvider.accept(documentId);
    }

    @ReactMethod
    public void isPrivacyPolicyAccepted(final String documentId, final Promise promise) {
        promise.resolve(this.privacyProvider.isAccepted(documentId));
    }

    @ReactMethod
    public void getAvailablePrivacyPolicyDocuments(final Promise promise) {
        this.getAvailableDocumentsWithProvider(this.privacyProvider, promise);
    }

    @ReactMethod
    public void getPrivacyPolicyLastCheckedTimestamp(final String country, final String language, final Promise promise) {
        Date timestamp = this.privacyProvider.getLastCheckedTimestamp(country, language);
        promise.resolve(isoDateFormatter.format(timestamp));
    }

    @ReactMethod
    public void getTimestampOfPrivacyPolicyConsent(final String documentId, final Promise promise) {
        Date timestamp = this.privacyProvider.getTimestampOfConsent(documentId);
        promise.resolve(isoDateFormatter.format(timestamp));
    }

    @ReactMethod
    public void getPrivacyPolicyDocument(final String country, final String language, final Promise promise) {
        this.getDocumentWithProvider(this.privacyProvider, country, language, promise);
    }

    @ReactMethod
    public void getAvailablePrivacyPolicyCountries(final ReadableArray countriesFromApp, final Promise promise) {
        final List<Country> countries = this.countryAssembler.createModelArray(countriesFromApp);
        final WritableArray countriesDTO = this.countryAssembler.createWritableDTOArray(this.privacyProvider.getAvailableCountries(countries));
        promise.resolve(countriesDTO);
    }


    // ---------------------------------------------- Terms of Use related methods ----------------------------------------------


    @ReactMethod
    public void initTermsOfUseProvider() {
        final ValueProvider valueProvider = new TermsOfUseProvider(this.reactContext);
        this.termsOfUseProvider = new TTPrivacyProviderBuilder(this.reactContext, null, valueProvider).onlineProvider(valueProvider).build();
    }

    @ReactMethod
    public void loadTermsOfUse(final ReadableMap country, final int viewTag) {
        this.loadDocumentWithProvider(this.termsOfUseProvider, country, viewTag);
    }

    @ReactMethod
    public void acceptTermsOfUse(final String documentId) {
        this.termsOfUseProvider.accept(documentId);
    }

    @ReactMethod
    public void isTermsOfUseAccepted(final String documentId, final Promise promise) {
        promise.resolve(this.termsOfUseProvider.isAccepted(documentId));
    }

    @ReactMethod
    public void getTermsOfUseLastCheckedTimestamp(final String country, final String language, final Promise promise) {
        Date timestamp = this.termsOfUseProvider.getLastCheckedTimestamp(country, language);
        promise.resolve(isoDateFormatter.format(timestamp));
    }

    @ReactMethod
    public void getTimestampOfTermsOfUseConsent(final String documentId, final Promise promise) {
        Date timestamp = this.termsOfUseProvider.getTimestampOfConsent(documentId);
        promise.resolve(isoDateFormatter.format(timestamp));
    }

    @ReactMethod
    public void getTermsOfUseDocument(final String country, final String language, final Promise promise) {
        this.getDocumentWithProvider(this.termsOfUseProvider, country, language, promise);
    }

    @ReactMethod
    public void getAvailableTermsOfUseDocuments(final Promise promise) {
        this.getAvailableDocumentsWithProvider(this.termsOfUseProvider, promise);
    }

    @ReactMethod
    public void getAvailableTermsOfUseCountries(final ReadableArray countriesFromApp, final Promise promise) {
        final List<Country> countries = this.countryAssembler.createModelArray(countriesFromApp);
        final WritableArray countriesDTO = this.countryAssembler.createWritableDTOArray(this.termsOfUseProvider.getAvailableCountries(countries));
        promise.resolve(countriesDTO);
    }


    // ---------------------------------------------- Activity Consent related methods ----------------------------------------------

    @ReactMethod
    public void initActivityConsentProvider(final String activityId) {
        this.activityProvider = new TTActivityProviderBuilder(this.reactContext, activityId,
                new ActivityConsentOfflineProvider(this.reactContext)).build();
        this.activityConsentAssembler = new ActivityConsentAssembler();
        this.onActivityDocumentLoadListeners = new SparseArray<>();
    }

    @ReactMethod
    public void loadActivityConsent(final ReadableMap country, final int viewTag) {
        final UIManagerModule uiManager = this.reactContext.getNativeModule(UIManagerModule.class);
        uiManager.addUIBlock(new UIBlock() {
            @Override
            public void execute(NativeViewHierarchyManager nativeViewHierarchyManager) {
                try {
                    final Country selectedCountry = countryAssembler.createModel(country);
                    final TTActivityComponent activityComponent = (TTActivityComponent) nativeViewHierarchyManager.resolveView(viewTag);
                    activityComponent.loadDocument(RNPrivacyComponentModule.this.activityProvider,
                            selectedCountry.getCode(), selectedCountry.getLanguage(),
                            getOnActivityDocumentLoadListener(activityComponent));
                } catch (IllegalViewOperationException e) {
                    Log.e(TAG, e.getMessage());
                }
            }
        });
    }

    @ReactMethod
    public void acceptActivityConsent(final String activityId) {
        this.activityProvider.accept(activityId);
    }

    @ReactMethod
    public void isActivityConsentAccepted(final String activityId, final Promise promise) {
        promise.resolve(this.activityProvider.isAccepted(activityId));
    }

    @ReactMethod
    public void getActivityConsentLastCheckedTimestamp(final String country, final String language, final Promise promise) {
        Date timestamp = this.activityProvider.getLastCheckedTimestamp(country, language);
        promise.resolve(isoDateFormatter.format(timestamp));
    }

    @ReactMethod
    public void getTimestampOfActivityConsent(final String activityId, final Promise promise) {
        Date timestamp = this.activityProvider.getTimestampOfConsent(activityId);
        promise.resolve(isoDateFormatter.format(timestamp));
    }

    @ReactMethod
    public void getActivityConsentDocument(final String country, final String language, final Promise promise) {
        final MetaActivityDocument activityDocument = (MetaActivityDocument) this.activityProvider.getDocument(country, language);
        if (activityDocument != null) {
            final WritableMap documentDTO = this.activityConsentAssembler.createWritableDTO(activityDocument);
            promise.resolve(documentDTO);
        } else {
            Log.w(TAG, "No activity consent found for country: " + country + ", language: " + language);
            promise.reject("404", "No activity consent found!");
        }
    }

    @ReactMethod
    public void getAvailableActivityConsentDocuments(final Promise promise) {
        final List<MetaActivityDocument> metaDocuments = (List<MetaActivityDocument>) this.activityProvider.getAvailableDocuments();
        if (metaDocuments != null) {
            final WritableArray readableArray = this.activityConsentAssembler.createWritableDTOArray(metaDocuments);
            promise.resolve(readableArray);
        } else {
            Log.w(TAG, "No available documents found!");
            promise.reject("404", "No available documents!");
        }
    }

    @ReactMethod
    public void getAvailableActivityConsentCountries(final ReadableArray countriesFromApp, final Promise promise) {
        final List<Country> countries = this.countryAssembler.createModelArray(countriesFromApp);
        final WritableArray countriesDTO = this.countryAssembler.createWritableDTOArray(this.activityProvider.getAvailableCountries(countries));
        promise.resolve(countriesDTO);
    }


    // ---------------------------------------------- Helper functions ----------------------------------------------


    /**
     * Retrieves the onDocumentLoadListener function which belongs to TTPrivacyComponent instance.
     *
     * @param privacyComponent - the instance to which the listener belongs
     * @return TTPrivacyComponent.OnLoadFinishedListener callback
     */
    private TTPrivacyComponent.OnLoadFinishedListener getOnDocumentLoadListener(final TTPrivacyComponent privacyComponent) {
        if (this.onDocumentLoadListeners.get(privacyComponent.getId()) == null) {
            final TTPrivacyComponent.OnLoadFinishedListener onLoadFinishedListener = new TTPrivacyComponent.OnLoadFinishedListener() {
                @Override
                public void onLoadFinished(Document document) {
                    reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(privacyComponent.getId(), TTComponentEvent.ON_DOCUMENT_LOADED.toString(), RNPrivacyComponentModule.this.documentAssembler.createWritableDTO(document));
                }
            };
            this.onDocumentLoadListeners.put(privacyComponent.getId(), onLoadFinishedListener);
        }
        return this.onDocumentLoadListeners.get(privacyComponent.getId());
    }

    /**
     * Retrieves the onDocumentLoadListener function which belongs to TTActivityComponent instance.
     *
     * @param activityComponent - the instance to which the listener belongs
     * @return TTActivityComponent.OnLoadFinishedListener callback
     */
    private TTActivityComponent.OnLoadFinishedListener getOnActivityDocumentLoadListener(final TTActivityComponent activityComponent) {
        if (this.onActivityDocumentLoadListeners.get(activityComponent.getId()) == null) {
            final TTActivityComponent.OnLoadFinishedListener onLoadFinishedListener = new TTActivityComponent.OnLoadFinishedListener() {
                @Override
                public void onLoadFinished(ActivityDocument document) {
                    reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(activityComponent.getId(), TTComponentEvent.ON_DOCUMENT_LOADED.toString(), RNPrivacyComponentModule.this.activityConsentAssembler.createWritableDTO(document));
                }
            };
            this.onActivityDocumentLoadListeners.put(activityComponent.getId(), onLoadFinishedListener);
        }
        return this.onActivityDocumentLoadListeners.get(activityComponent.getId());
    }

    /**
     * Retrieves a privacy policy or terms of use document for the given country and language in a promise.
     *
     * @param provider  - to which the document belongs (privacyProvider/termsOfUseProvider)
     * @param country   - to which country the document belongs
     * @param language  - on which language the document should be retrieved
     * @param promise   - resolves with the document if it is found, rejects otherwise
     */
    private void getDocumentWithProvider(final TTPrivacyProvider provider, final String country, final String language, final Promise promise) {
        final Document document = provider.getDocument(country, language);
        if (document != null) {
            final WritableMap documentDTO = this.documentAssembler.createWritableDTO(document);
            promise.resolve(documentDTO);
        } else {
            Log.w(TAG, "No document found for country: " + country + ", language: " + language);
            promise.reject("404", "No document found!");
        }
    }

    /**
     * Retrieves all the documents for the given provider in a promise.
     *
     * @param provider  - to which the documents belong (privacyProvider/termsOfUseProvider)
     * @param promise   - resolves with the documents if they exist, rejects otherwise
     */
    private void getAvailableDocumentsWithProvider(final TTPrivacyProvider provider, final Promise promise) {
        final List<MetaDocument> metaDocuments = provider.getAvailableDocuments();
        if (metaDocuments != null) {
            final WritableArray readableArray = this.documentAssembler.createWritableDTOArray(metaDocuments);
            promise.resolve(readableArray);
        } else {
            Log.w(TAG, "No available documents found!");
            promise.reject("404", "No available documents!");
        }
    }

    /**
     * Loads the privacy policy or the terms of use document in the WebView.
     *
     * @param provider  - to which the document belongs (privacyProvider/termsOfUseProvider)
     * @param country   - to which country the document is needed
     * @param viewTag   - the id of the TTPrivacyComponent on which the document should be loaded
     */
    private void loadDocumentWithProvider(final TTPrivacyProvider provider, final ReadableMap country, final int viewTag) {
        final UIManagerModule uiManager = this.reactContext.getNativeModule(UIManagerModule.class);
        uiManager.addUIBlock(new UIBlock() {
            @Override
            public void execute(NativeViewHierarchyManager nativeViewHierarchyManager) {
                try {
                    final Country selectedCountry = countryAssembler.createModel(country);
                    final TTPrivacyComponent privacyComponent = (TTPrivacyComponent) nativeViewHierarchyManager.resolveView(viewTag);
                    privacyComponent.loadDocument(provider, selectedCountry.getCode(), selectedCountry.getLanguage(), getOnDocumentLoadListener(privacyComponent));
                } catch (IllegalViewOperationException e) {
                    Log.e(TAG, e.getMessage());
                }
            }
        });
    }
}